#ifndef INCLUDE__BasicBlockWidget_hxx
#define INCLUDE__BasicBlockWidget_hxx

#include "gui/qt.hxx"
#include <vector>
#include <cassert>
#include <tuple>
#include <array>
#include <memory>
#include <log4cxx/logger.h>

class Mainwindow;
class CustomQGraphicsTextItem;

class BasicBlock;

class RenameFunctionEvent;

class BasicBlockWidget : public QObject, public QGraphicsItem
{
	Q_OBJECT
	friend class CustomQGraphicsTextItem;
public:
	BasicBlockWidget(const QString& name, BasicBlock * block, Mainwindow * mainwindow);

	void addItem(uint8_t* bytes, size_t num_bytes, QString line, const QString& href);
	void paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
	           QWidget *widget);
	QRectF boundingRect() const;
	std::array<QPointF, 3> getExits() const;

	void mouseMoveEvent(QGraphicsSceneMouseEvent * event)
		{ QGraphicsItem::mouseMoveEvent(event); scene()->update(); }

	QPointF getEntry() const
		{ return mapToScene(QPointF(width/2, 0)); }

	void addPrevious(BasicBlockWidget * widget)
		{ previous.push_back(widget); }

	void addNext(BasicBlockWidget * left, BasicBlockWidget * right)
		{ next[0] = left; next[1] = right; }

	BasicBlockWidget ** getNext()
		{ return next; }

	QString getName() const
		{ return name; }
private:
	void updateFunctionName(RenameFunctionEvent* event);

	uint32_t width, height;
	QString name;
	std::unique_ptr<QGraphicsTextItem> _widget;
	QTextTable* _table;
	BasicBlock* block;
	Mainwindow* mainwindow;
	std::vector<BasicBlockWidget*> previous;
	BasicBlockWidget* next[2];
	log4cxx::LoggerPtr logger;
};

#endif
